@@ -1,6 +1,6 @@
 package version
 
 var (
-	Release = "1.24.1-dev"
+	Release = "1.25.0"
 	GitHash string
 )
